@@ -1,5 +1,14 @@
+"""
+@Author  : Likianta <likianta@foxmail.com>
+@Module  : chinese_name_processor.py
+@Created : 2018-00-00
+@Updated : 2020-08-09
+@Version : 1.4.18
+@Desc    :
+"""
 import re
 
+from ._typings import *
 from .read_and_write import read_file_by_line, read_json
 from .tree_and_trie import Trie
 
@@ -9,11 +18,9 @@ class Lexicon:
     def __init__(self):
         from os.path import split as osplit
         
-        selfile = __file__.replace('\\', '/')
-        res_dir = osplit(selfile)[0] + '/resource/'
-        # 注: res_dir 是绝对路径定位的.
-        
-        self.res_dir = res_dir
+        self_file = __file__.replace('\\', '/')
+        resrc_dir = osplit(self_file)[0] + '/resource'  # resource dir
+        self.resrc_dir = resrc_dir
         
         """
         metadata:
@@ -22,316 +29,92 @@ class Lexicon:
             韦氏拼音表 => 韦氏拼音表_标准集
             韦氏拼音表 + 非标准韦氏拼音表 => 韦氏拼音表_扩展集
         其中, 等式左侧的 "拼音表", "非标准拼音表", "韦氏拼音表", "非标准韦氏拼音
-        表" 是由人工维护和定期更新的, 等式右侧的 "拼音表_标准集", "拼音表_扩展
-        集", "韦氏拼音表_标准集", "韦氏拼音表_扩展集" 包括后面会涉及到的前缀树,
-        后缀树等衍生品, 都是由脚本根据 metadata 组合生成的 (脚本见 metadata
-        /lexicon_generator.py).
+         表" 是由人工维护和定期更新的, 等式右侧的 "拼音表_标准集", "拼音表_扩展
+         集", "韦氏拼音表_标准集", "韦氏拼音表_扩展集" 包括后面会涉及到的前缀树,
+         后缀树等衍生品, 都是由脚本根据 metadata 组合生成的 (脚本见 metadata/
+         lexicon_generator.py).
         
-        下面的 self.res 就是脚本生成的资源文件路径字典.
+        下面的 self.resrc 就是脚本生成的资源文件路径字典.
         """
-        # res: resources, 该字典是一个三级结构. 领域 > 数据集 > 文件类型
-        self.res = {
+        # 该字典是一个三级结构. 领域 > 数据集 > 文件类型.
+        self.resrc = {
             'pinyin'      : {
                 'standard': {
-                    'plain'  : res_dir + '拼音表_标准集.txt',
-                    'trie'   : res_dir + '拼音表_标准集_前缀树.json',
-                    'postrie': res_dir + '拼音表_标准集_后缀树.json',
+                    'plain'  : f'{resrc_dir}/拼音表_标准集.txt',
+                    'trie'   : f'{resrc_dir}/拼音表_标准集_前缀树.json',
+                    'postrie': f'{resrc_dir}/拼音表_标准集_后缀树.json',
                 },
                 'extend'  : {
-                    'plain'  : res_dir + '拼音表_扩展集.txt',
-                    'trie'   : res_dir + '拼音表_扩展集_前缀树.json',
-                    'postrie': res_dir + '拼音表_扩展集_后缀树.json',
+                    'plain'  : f'{resrc_dir}/拼音表_扩展集.txt',
+                    'trie'   : f'{resrc_dir}/拼音表_扩展集_前缀树.json',
+                    'postrie': f'{resrc_dir}/拼音表_扩展集_后缀树.json',
                 }
             },
             'wade'        : {
                 'standard': {
-                    'plain'  : res_dir + '韦氏拼音表_标准集.txt',
-                    'trie'   : res_dir + '韦氏拼音表_标准集_前缀树.json',
-                    'postrie': res_dir + '韦氏拼音表_标准集_后缀树.json',
+                    'plain'  : f'{resrc_dir}/韦氏拼音表_标准集.txt',
+                    'trie'   : f'{resrc_dir}/韦氏拼音表_标准集_前缀树.json',
+                    'postrie': f'{resrc_dir}/韦氏拼音表_标准集_后缀树.json',
                 },
                 'extend'  : {
-                    'plain'  : res_dir + '韦氏拼音表_扩展集.txt',
-                    'trie'   : res_dir + '韦氏拼音表_扩展集_前缀树.json',
-                    'postrie': res_dir + '韦氏拼音表_扩展集_后缀树.json',
+                    'plain'  : f'{resrc_dir}/韦氏拼音表_扩展集.txt',
+                    'trie'   : f'{resrc_dir}/韦氏拼音表_扩展集_前缀树.json',
+                    'postrie': f'{resrc_dir}/韦氏拼音表_扩展集_后缀树.json',
                 }
             },
             'pinyin+wade' : {
                 'ss': {
-                    'plain'  : res_dir
-                               + '拼音表(标准集)+韦氏拼音表(标准集).txt',
-                    'trie'   : res_dir
-                               + '拼音表(标准集)+韦氏拼音表(标准集)_前缀树.json',
-                    'postrie': res_dir
-                               + '拼音表(标准集)+韦氏拼音表(标准集)_后缀树.json',
+                    'plain'  : f'{resrc_dir}/拼音表(标准集)+韦氏拼音表(标准集).txt',
+                    'trie'   : f'{resrc_dir}/拼音表(标准集)+韦氏拼音表(标准集)_前缀树.json',
+                    'postrie': f'{resrc_dir}/拼音表(标准集)+韦氏拼音表(标准集)_后缀树.json',
                 },
                 'se': {
-                    'plain'  : res_dir
-                               + '拼音表(标准集)+韦氏拼音表(扩展集).txt',
-                    'trie'   : res_dir
-                               + '拼音表(标准集)+韦氏拼音表(扩展集)_前缀树.json',
-                    'postrie': res_dir
-                               + '拼音表(标准集)+韦氏拼音表(扩展集)_后缀树.json',
+                    'plain'  : f'{resrc_dir}/拼音表(标准集)+韦氏拼音表(扩展集).txt',
+                    'trie'   : f'{resrc_dir}/拼音表(标准集)+韦氏拼音表(扩展集)_前缀树.json',
+                    'postrie': f'{resrc_dir}/拼音表(标准集)+韦氏拼音表(扩展集)_后缀树.json',
                 },
                 'es': {
-                    'plain'  : res_dir
-                               + '拼音表(扩展集)+韦氏拼音表(标准集).txt',
-                    'trie'   : res_dir
-                               + '拼音表(扩展集)+韦氏拼音表(标准集)_前缀树.json',
-                    'postrie': res_dir
-                               + '拼音表(扩展集)+韦氏拼音表(标准集)_后缀树.json',
+                    'plain'  : f'{resrc_dir}/拼音表(扩展集)+韦氏拼音表(标准集).txt',
+                    'trie'   : f'{resrc_dir}/拼音表(扩展集)+韦氏拼音表(标准集)_前缀树.json',
+                    'postrie': f'{resrc_dir}/拼音表(扩展集)+韦氏拼音表(标准集)_后缀树.json',
                     
                 },
                 'ee': {
-                    'plain'  : res_dir
-                               + '拼音表(扩展集)+韦氏拼音表(扩展集).txt',
-                    'trie'   : res_dir
-                               + '拼音表(扩展集)+韦氏拼音表(扩展集)_前缀树.json',
-                    'postrie': res_dir
-                               + '拼音表(扩展集)+韦氏拼音表(扩展集)_后缀树.json',
+                    'plain'  : f'{resrc_dir}/拼音表(扩展集)+韦氏拼音表(扩展集).txt',
+                    'trie'   : f'{resrc_dir}/拼音表(扩展集)+韦氏拼音表(扩展集)_前缀树.json',
+                    'postrie': f'{resrc_dir}/拼音表(扩展集)+韦氏拼音表(扩展集)_后缀树.json',
                 }
             },
             'chinese_name': {
                 'pinyin': {
-                    'single': res_dir + '中国姓氏_拼音_单姓.txt',
-                    'double': res_dir + '中国姓氏_拼音_复姓.txt',
-                    'wade'  : res_dir + '中国姓氏_拼音_韦氏.txt',
+                    'single': f'{resrc_dir}/中国姓氏_拼音_单姓.txt',
+                    'double': f'{resrc_dir}/中国姓氏_拼音_复姓.txt',
+                    'wade'  : f'{resrc_dir}/中国姓氏_拼音_韦氏.txt',
                 },
                 'hanz'  : {
-                    'single': res_dir + '中国姓氏_汉字_单姓.txt',
-                    'double': res_dir + '中国姓氏_汉字_复姓.txt',
+                    'single': f'{resrc_dir}/中国姓氏_汉字_单姓.txt',
+                    'double': f'{resrc_dir}/中国姓氏_汉字_复姓.txt',
                     # 韦氏没有汉字
                 }
             }
         }
     
     def get(self, data_type, data_spec, data_fmt):
-        # e.g. lexicon.get('pinyin', 'standard', 'trie')
-        return self.res[data_type][data_spec][data_fmt]
+        # Use case: e.g. lexicon.get('pinyin', 'standard', 'trie')
+        return self.resrc[data_type][data_spec][data_fmt]
 
 
 lexicon = Lexicon()
 
 
-# ------------------------------------------------
-
-class PinyinProcessor:
-    trie = None
-    word_pattern = None
-    pinyin_patterns = None
-    pinyin_patterns_4_name = None
-    
-    # ------------------------------------------------
-    
-    def __init__(self, wade_mode=False, trie_mode='postrie'):
-        self.wade_mode = wade_mode
-        self.pinyin_cutter = PinyinCutter(wade_mode, trie_mode)
-        
-        self._init_tries(wade_mode)
-        self._init_pinyin_patterns(wade_mode)
-    
-    def _init_tries(self, wade_mode):
-        self.trie = Trie()
-        if wade_mode:
-            self.trie.load(lexicon.get('pinyin+wade', 'se', 'trie'))
-            # -> 拼音表(标准集)+韦氏拼音表(扩展集)_前缀树.json
-        else:
-            self.trie.load(lexicon.get('pinyin', 'standard', 'trie'))
-            # -> 拼音表_标准集_前缀树.json
-    
-    def _init_pinyin_patterns(self, wade_mode):
-        """
-        读取中国姓氏拼音列表
-        """
-        self.word_pattern = re.compile(r'\w+')
-        # 该匹配式用于匹配人名中所有连续字符串, 从而剔除掉逗号, 引号, 句点之类的
-        # 干扰因素
-        
-        self.pinyin_patterns = {
-            'pinyin'     : read_file_by_line(
-                lexicon.get('pinyin', 'extend', 'plain')
-            ),
-            'wade'       : read_file_by_line(
-                lexicon.get('wade', 'extend', 'plain')
-            ),
-            'pinyin+wade': read_file_by_line(
-                lexicon.get('pinyin+wade', 'ee', 'plain')
-            )
-        }
-        
-        self.pinyin_patterns_4_name = {
-            'single'    : read_file_by_line(
-                lexicon.get('chinese_name', 'pinyin', 'single')
-            ),  # -> 中国姓氏_拼音_单姓.txt
-            'double'    : read_file_by_line(
-                lexicon.get('chinese_name', 'pinyin', 'double')
-            ),  # -> 中国姓氏_拼音_复姓.txt
-            'wade'      : read_file_by_line(
-                lexicon.get('chinese_name', 'pinyin', 'wade')
-            ),  # -> 中国姓氏_拼音_韦氏.txt
-            'exceptions': (
-                # 'ai',
-                'al',
-                'de la', 'de', 'der', 'des', 'di',
-                'el',
-                'fits', 'fitz', 'ftz',
-                'jr', 'jun', 'junior',
-                'la', 'le',
-                'mac', 'mc',
-                # 'o',
-                'van', 'von',
-            )
-        }
-        
-        if wade_mode:
-            self.pinyin_patterns['default'] = \
-                self.pinyin_patterns['pinyin+wade']
-            self.pinyin_patterns_4_name['default'] = [
-                *self.pinyin_patterns_4_name['single'],
-                *self.pinyin_patterns_4_name['double'],
-                *self.pinyin_patterns_4_name['wade'],
-            ]
-        else:
-            self.pinyin_patterns['default'] = self.pinyin_patterns['pinyin']
-            self.pinyin_patterns_4_name['default'] = [
-                *self.pinyin_patterns_4_name['single'],
-                *self.pinyin_patterns_4_name['double'],
-            ]
-    
-    # ------------------------------------------------ pinyin patterns
-    
-    def is_chinese(self, name: str) -> bool:
-        """
-        给定一个名字, 根据他的姓氏来判断是不是中国人.
-        
-        ARGS:
-            name: 您可以传整个姓名, 也可以只传姓氏. 姓名只支持 'lastname,
-                firstname' 格式的姓名. 当传入姓名时, 判断可能会更加准确. 例如,
-                如果只传入 "jan", 那么在 self.wade_mode is True 的情况下, 会判断
-                为 True; 而传入 "jan, edward" 和 "jan, ping xin", 则会分别判断为
-                False 和 True.
-        
-        NOTE:
-            1. name 全小写
-            2. 当传入全名时, 必须使用姓前名后且姓和名之间用英文逗号加空格分隔,
-                也就是 'lastname, firstname' 的形式. 例如 "zhang, san"
-            3. 由于针对全名的判断 (e.g. "jan, edward" 和 "jan, ping xin") 并不完
-                善, 所以目前 (2019年12月3日) 不启用
-        
-        IO: da         -> True
-            hu         -> True
-            baker      -> False
-            ouyang     -> True
-            ou yang    -> True
-            von briton -> False
-        """
-        name = name.lower()
-        
-        if ', ' in name:
-            lastname, firstname = name.split(', ', 1)
-            # 'zhang, tiejun' -> 'zhang', 'tiejun'
-        else:
-            lastname, firstname = name, ''
-        # NOTE: 由于针对 firstname 的判断并不完善, 且容易存在误判, 所以
-        # firstname 变量暂时 (2019年12月3日) 没有任何作用.
-        
-        if lastname in self.pinyin_patterns_4_name['double']:
-            # e.g. lastname = 'ouyang' -> True
-            return True
-        
-        if ' ' in lastname and any(
-                lastname.startswith(x)
-                for x in self.pinyin_patterns_4_name['exceptions']
-        ):  # e.g. lastname = 'von briton'
-            return False
-        
-        """
-        NOTE: 原先是打算用 self.pinyin_patterns_4_name 的 pinyin 和 wade 的值来
-        判断的, 后来发现有些罕见的姓, 虽然符合拼音或韦氏拼音的构造规则, 但由于不
-        在 self.pinyin_patterns_4_name 范围内, 所以会被误判为非中国人名. 为避免
-        这种情况, 不再使用 self.pinyin_patterns_4_name 提供的 pinyin 和 wade, 改
-        用 self.pinyin_patterns 的 'pinyin+wade'.
-        """
-        if self.wade_mode:
-            return lastname in self.pinyin_patterns['pinyin+wade']
-        else:
-            return lastname in self.pinyin_patterns['pinyin']
-    
-    # ------------------------------------------------ tries
-    
-    def is_pinyin(self, name, strict=False):
-        """
-        判断名字是英文单词还是拼音的形式.
-        
-        IO:
-            | gerald    | False | 识别出英文名         |
-            | ming      | True  | 识别出拼音           |
-            | lee       | True  | 支持韦氏拼音         |
-            | guangshan | True  | 支持拼接在一起的拼音 |
-        NOTE:
-            1. 请确保 name 是全小写, 否则 self.trie 无法正常工作
-            2. 建议 name 只传入单词
-            2. 可能存在误识别. 例如 "sheer" 会被认作 ["she", "er"] -> True. 您可
-               以通过传递 strict=True 参数来关闭分词模式
-            3. 对于中英混合的情况, 会返回 False. 例如 "zhen cool" -> False
-        """
-        if name in self.pinyin_patterns['default']:
-            return True
-        if strict:
-            return False
-        name1 = ''.join(self.word_pattern.findall(name))
-        name2 = ''.join(self.trie.findall(name1))
-        return name1 == name2
-    
-    def cut_pinyin(self, name: str) -> list:
-        """
-        对中国人名拼音的切分.
-        
-        特点:
-            1. 支持对普通拼音的切分
-            2. 支持对中国复姓的切分
-            3. 支持对有歧义的发音的切分
-            4. 支持单音节词 (比如 a, o, e, er, ai, ang 等) 的发音匹配
-            5. 当 wade_mode 为 True 时, 支持韦氏拼音的切分
-            6. 即便在切分失败的情况下, 也能返回较理想的切分结果
-                示例:
-                    'li, gerald adam'
-                    -> 英文单词无法正常切分
-                    -> 返回 ['li', 'gerald', 'adam']
-                    'fan, taiping d.'
-                    -> 'd' 无法正常切分
-                    -> 返回 ['fan', 'tai', 'ping', 'd']
-            
-        效果:
-            一般人名 (朱自清): 'zhuziqing' -> ['zhu', 'zi', 'qing']
-            复姓 (欧阳修): 'ouyangxiu' -> ['ou', 'yang', 'xiu']
-            歧义组合 (柴凤安): 'chaifengan' -> ['chai', 'feng', 'an'] (注: 不支
-            持切出 chai fen gan)
-            单音节词1 (刘酬安): 'liuchouan' -> ['liu', 'chou', 'an']
-            单音节词2 (刘马鄂): 'liumae' -> ['liu', 'ma', 'e']
-    
-        注:
-            对于某些歧义组合, 只支持最大向前匹配. 例如:
-                "fangan" -> ["fang", "an"]
-            wade_mode 可能引发切分错误. 错误主要发生在韦氏拼音的韵母的最后一个字
-            母和下一个字的声母的首字母冲突, 例如:
-                "wang, wiehen" -> ["wang", "wieh", "en"] (切分不当)
-    
-        参考:
-            https://blog.csdn.net/lemon_tree12138/article/details/49074809
-            #commentBox
-            https://zhidao.baidu.com/question/21799136.html
-        
-        """
-        return self.pinyin_cutter.main(name)
-
+# ------------------------------------------------------------------------------
 
 class PinyinCutter:
     
     def __init__(self, wade_mode=False, trie_mode='postrie'):
         """
-        ARGS:
-            wade_mode: bool
-            trie_mode: str. 'postrie'/'trie'
+        :param wade_mode: bool
+        :param trie_mode: str. 'postrie'/'trie'.
                 'postrie': 反向最大匹配切词法. 例如:
                     'yangan' -> ['yan', 'gan']
                     'daniel' -> ['da', 'nie', 'l']
@@ -339,7 +122,7 @@ class PinyinCutter:
                     'yangan' -> ['yang', 'an']
                     'daniel' -> ['dan', 'i', 'e', 'l']
         """
-        self.obsv_mode = 'trie'  # obsv: observer  # DEL: no usage
+        self.obsv_mode = 'trie'  # obsv: observer (no usage)
         self.trie_mode = trie_mode
         
         if wade_mode:
@@ -388,9 +171,8 @@ class PinyinCutter:
         
         return out
     
-    def cut_segs_1(self, word) -> list:
-        """
-        适用于 trie_mode = 'trie'.
+    def cut_segs_1(self, word: str) -> list:
+        """ 适用于 trie_mode = 'trie'.
         
         IO:
             'wang' -> ['wang']
@@ -428,8 +210,7 @@ class PinyinCutter:
         return out
     
     def cut_segs_2(self, word: str) -> list:
-        """
-        适用于 trie_mode = 'postrie'. 这是默认的选择.
+        """ 适用于 trie_mode = 'postrie'. 这是默认的选择.
         
         IO:
             'wang' -> ['wang']
@@ -516,7 +297,212 @@ class PinyinCutter:
             return ' '.join(out)
 
 
-# ------------------------------------------------
+class PinyinProcessor:
+    pinyin_cutter: PinyinCutter
+    pinyin_patterns: dict
+    pinyin_patterns_4_name: dict
+    wade_mode: bool
+    word_pattern: re.Pattern
+    trie: Trie
+    
+    def __init__(self, wade_mode=False, trie_mode='postrie'):
+        self.wade_mode = wade_mode
+        self.pinyin_cutter = PinyinCutter(wade_mode, trie_mode)
+        self._init_tries(wade_mode)
+        self._init_pinyin_patterns(wade_mode)
+    
+    def _init_tries(self, wade_mode: bool):
+        self.trie = Trie()
+        if wade_mode:
+            self.trie.load(lexicon.get('pinyin+wade', 'se', 'trie'))
+            # -> 拼音表(标准集)+韦氏拼音表(扩展集)_前缀树.json
+        else:
+            self.trie.load(lexicon.get('pinyin', 'standard', 'trie'))
+            # -> 拼音表_标准集_前缀树.json
+    
+    def _init_pinyin_patterns(self, wade_mode: bool):
+        """ 读取中国姓氏拼音列表. """
+        # self.word_pattern 该匹配式用于匹配人名中所有连续字符串, 从而剔除掉逗
+        #  号, 引号, 句点之类的干扰因素.
+        self.word_pattern = re.compile(r'\w+')
+        
+        self.pinyin_patterns = {
+            'pinyin'     : read_file_by_line(
+                lexicon.get('pinyin', 'extend', 'plain')
+            ),
+            'wade'       : read_file_by_line(
+                lexicon.get('wade', 'extend', 'plain')
+            ),
+            'pinyin+wade': read_file_by_line(
+                lexicon.get('pinyin+wade', 'ee', 'plain')
+            )
+        }
+        
+        self.pinyin_patterns_4_name = {
+            'single'    : read_file_by_line(
+                lexicon.get('chinese_name', 'pinyin', 'single')
+            ),  # -> 中国姓氏_拼音_单姓.txt
+            'double'    : read_file_by_line(
+                lexicon.get('chinese_name', 'pinyin', 'double')
+            ),  # -> 中国姓氏_拼音_复姓.txt
+            'wade'      : read_file_by_line(
+                lexicon.get('chinese_name', 'pinyin', 'wade')
+            ),  # -> 中国姓氏_拼音_韦氏.txt
+            'exceptions': (
+                # 'ai',
+                'al',
+                'de la', 'de', 'der', 'des', 'di',
+                'el',
+                'fits', 'fitz', 'ftz',
+                'jr', 'jun', 'junior',
+                'la', 'le',
+                'mac', 'mc',
+                # 'o',
+                'van', 'von',
+            )
+        }
+        
+        if wade_mode:
+            self.pinyin_patterns['default'] = \
+                self.pinyin_patterns['pinyin+wade']
+            self.pinyin_patterns_4_name['default'] = [
+                *self.pinyin_patterns_4_name['single'],
+                *self.pinyin_patterns_4_name['double'],
+                *self.pinyin_patterns_4_name['wade'],
+            ]
+        else:
+            self.pinyin_patterns['default'] = self.pinyin_patterns['pinyin']
+            self.pinyin_patterns_4_name['default'] = [
+                *self.pinyin_patterns_4_name['single'],
+                *self.pinyin_patterns_4_name['double'],
+            ]
+
+    # --------------------------------------------------------------------------
+    # pinyin patterns
+    
+    def is_chinese(self, name: str) -> bool:
+        """ 给定一个名字, 根据他的姓氏来判断是不是中国人.
+        
+        NOTE:
+            1. name 全小写.
+            2. 当传入全名时, 必须使用姓前名后且姓和名之间用英文逗号加空格分隔,
+                也就是 'lastname, firstname' 的形式. 例如 "zhang, san".
+            3. 由于针对全名的判断 (e.g. "jan, edward" 和 "jan, ping xin") 并不完
+                善, 所以目前 (2019年12月3日) 不启用.
+        
+        IO: da         -> True
+            hu         -> True
+            baker      -> False
+            ouyang     -> True
+            ou yang    -> True
+            von briton -> False
+            
+        :param name: 您可以传整个姓名, 也可以只传姓氏. 姓名只支持 'lastname,
+         firstname' 格式的姓名. 当传入姓名时, 判断可能会更加准确. 例如, 如果只传
+         入 "jan", 那么在 `self.wade_mode is True` 的情况下, 会判断为 True; 而传
+         入 "jan, edward" 和 "jan, ping xin", 则会分别判断为 False 和 True.
+        """
+        name = name.lower()
+        
+        if ', ' in name:
+            lastname, firstname = name.split(', ', 1)
+            # 'zhang, tiejun' -> 'zhang', 'tiejun'
+        else:
+            lastname, firstname = name, ''
+        # NOTE: 由于针对 firstname 的判断并不完善, 且容易存在误判, 所以
+        # firstname 变量暂时 (2019年12月3日) 没有任何作用.
+        
+        if lastname in self.pinyin_patterns_4_name['double']:
+            # e.g. lastname = 'ouyang' -> True
+            return True
+        
+        if ' ' in lastname and any(
+                lastname.startswith(x)
+                for x in self.pinyin_patterns_4_name['exceptions']
+        ):  # e.g. lastname = 'von briton'
+            return False
+        
+        """
+        NOTE: 原先是打算用 self.pinyin_patterns_4_name 的 pinyin 和 wade 的值来
+         判断的, 后来发现有些罕见的姓, 虽然符合拼音或韦氏拼音的构造规则, 但由于
+         不在 self.pinyin_patterns_4_name 范围内, 所以会被误判为非中国人名. 为避
+         免这种情况, 不再使用 self.pinyin_patterns_4_name 提供的 pinyin 和 wade,
+         改用 self.pinyin_patterns 的 'pinyin+wade'.
+        """
+        if self.wade_mode:
+            return lastname in self.pinyin_patterns['pinyin+wade']
+        else:
+            return lastname in self.pinyin_patterns['pinyin']
+
+    # --------------------------------------------------------------------------
+    # tries
+    
+    def is_pinyin(self, name, strict=False):
+        """ 判断名字是英文单词还是拼音的形式.
+        
+        IO: | gerald    | False | 识别出英文名         |
+            | ming      | True  | 识别出拼音           |
+            | lee       | True  | 支持韦氏拼音         |
+            | guangshan | True  | 支持拼接在一起的拼音 |
+            
+        NOTE:
+            1. 请确保 name 是全小写, 否则 self.trie 无法正常工作.
+            2. 建议 name 只传入单词.
+            3. 可能存在误识别. 例如 "sheer" 会被认作 ["she", "er"] -> True. 您可
+               以通过传递 strict=True 参数来关闭分词模式.
+            4. 对于中英混合的情况, 会返回 False. 例如 "zhen cool" -> False.
+        """
+        if name in self.pinyin_patterns['default']:
+            return True
+        elif strict:
+            return False
+        else:
+            name1 = ''.join(self.word_pattern.findall(name))
+            name2 = ''.join(self.trie.findall(name1))
+            return name1 == name2
+    
+    def cut_pinyin(self, name: str) -> list:
+        """ 对中国人名拼音的切分.
+        
+        特点:
+            1. 支持对普通拼音的切分
+            2. 支持对中国复姓的切分
+            3. 支持对有歧义的发音的切分
+            4. 支持单音节词 (比如 a, o, e, er, ai, ang 等) 的发音匹配
+            5. 当 wade_mode 为 True 时, 支持韦氏拼音的切分
+            6. 即便在切分失败的情况下, 也能返回较理想的切分结果
+                示例:
+                    'li, gerald adam'
+                    -> 英文单词无法正常切分
+                    -> 返回 ['li', 'gerald', 'adam']
+                    'fan, taiping d.'
+                    -> 'd' 无法正常切分
+                    -> 返回 ['fan', 'tai', 'ping', 'd']
+            
+        效果:
+            一般人名 (朱自清): 'zhuziqing' -> ['zhu', 'zi', 'qing']
+            复姓 (欧阳修): 'ouyangxiu' -> ['ou', 'yang', 'xiu']
+            歧义组合 (柴凤安): 'chaifengan' -> ['chai', 'feng', 'an'] (注: 不支
+             持切出 chai fen gan)
+            单音节词1 (刘酬安): 'liuchouan' -> ['liu', 'chou', 'an']
+            单音节词2 (刘马鄂): 'liumae' -> ['liu', 'ma', 'e']
+    
+        注:
+            对于某些歧义组合, 只支持最大向前匹配. 例如:
+                "fangan" -> ["fang", "an"]
+            wade_mode 可能引发切分错误. 错误主要发生在韦氏拼音的韵母的最后一个字
+             母和下一个字的声母的首字母冲突, 例如:
+                "wang, wiehen" -> ["wang", "wieh", "en"] (切分不当)
+    
+        参考:
+            https://blog.csdn.net/lemon_tree12138/article/details/49074809
+             #commentBox
+            https://zhidao.baidu.com/question/21799136.html
+        """
+        return self.pinyin_cutter.main(name)
+
+
+# ------------------------------------------------------------------------------
 
 class HanzProcessor:
     
@@ -529,10 +515,9 @@ class HanzProcessor:
         )  # -> 中国姓氏_汉字_复姓.txt
         self.is_letter = re.compile('[a-zA-Z]')
     
-    def is_chinese(self, name: str):
+    def is_chinese(self, name: str) -> bool:
         if self.is_letter.findall(name):
             return False
-        
         if 2 <= len(name) <= 4:
             if name[0] in self.lastname_single \
                     or name[0:2] in self.lastname_double:
@@ -540,9 +525,9 @@ class HanzProcessor:
                 return True
         return False
     
-    def chinese_name_to_pinyin(self, name: str, conjunction=''):
-        """
-        将中国人名 (中文名) 转换成拼音.
+    def chinese_name_to_pinyin(self, name: str,
+                               conjunction='') -> Union[str, List[str]]:
+        """ 将中国人名 (中文名) 转换成拼音.
         
         IO:
             conjunction = ''
@@ -563,22 +548,17 @@ class HanzProcessor:
             1. 支持多音字姓氏的简易转换, 以规避误发音的情况
             2. 针对常见复姓做了调整, 可以正确地把复姓当做姓氏
         """
+        # noinspection PyUnresolvedReferences
         from pypinyin import lazy_pinyin
-        
         name = self.polyphonic_conversion(name)
-        
         pinyin = lazy_pinyin(name)
         # '张磊' -> ['zhang', 'lei']
-        
         if conjunction == 'list':
             return pinyin
         
-        # ------------------------------------------------
-        
         compound_surnames = (
             "欧阳", "诸葛", "东方", "端木", "上官", "公孙", "司马"
         )
-        
         if any(x for x in compound_surnames if x in name):
             cut_point = 2
         else:
@@ -592,24 +572,21 @@ class HanzProcessor:
                                          for x in pinyin[:cut_point]))
             firstname = conjunction.join((x.title()
                                           for x in pinyin[cut_point:]))
-        
         return f'{lastname}, {firstname}'
     
     @staticmethod
-    def polyphonic_conversion(name):
-        """
-        将百家姓中的多音字转换为同音的非多音字.
+    def polyphonic_conversion(name: str) -> str:
+        """ 将百家姓中的多音字转换为同音的非多音字.
         
-        百家姓里的多音字:
-            https://zhidao.baidu.com/question/188597315.html?qbl=relate_question
-            _0&word=%B0%D9%BC%D2%D0%D5%20%B6%E0%D2%F4%D7%D6
+        百家姓里的多音字: https://zhidao.baidu.com/question/188597315.html?qbl=
+         relate_question_0&word=%B0%D9%BC%D2%D0%D5%20%B6%E0%D2%F4%D7%D6
         
         注意:
             1. "汤" -> "商" 变换未认证, 所以没收录
             2. "黑" -> "he" 情况比较复杂, 有些地方的人还是读成 "hei", 所以不收录
             2. 一般来说, "长" 在人名中经常被读 "chang", 例如 "左长宁", "杨虎长",
-            另外, 也有一个例外情况是 "杨成长", 这时适合读 "zhang", 可根据这种规
-            则来调节
+               另外, 也有一个例外情况是 "杨成长", 这时适合读 "zhang", 可根据这种
+               规则来调节
         """
         
         polyphonic = {
@@ -617,7 +594,6 @@ class HanzProcessor:
             '盖': '葛', '缪': '妙', '区': '欧', '朴': '飘', '仇': '求',
             '单': '善', '折': '舌', '冼': '显', '解': '谢', '员': '蕴',
             '曾': '增', '查': '渣', '翟': '宅',
-            # ------------------------------------------------
             # 其他常用字 (非姓氏, 常见于名)
             '伽': '加', '长': '常',
         }
