@@ -2,8 +2,8 @@
 @Author  : Likianta <likianta@foxmail.com>
 @Module  : lk_logger.py
 @Created : 2018-00-00
-@Updated : 2020-08-05
-@Version : 3.5.15
+@Updated : 2020-08-21
+@Version : 3.5.17
 @Desc    :
 
 Abbreviations:
@@ -124,7 +124,7 @@ class CallerFinder:
         return filepath, lineno, function
     
     def find_caller_by_hierarchy(self, hierarchy):
-        """ find the source caller related to the target hierarchy.
+        """ Find the source caller related to the target hierarchy.
         
         caller frame hierarchy:
             | caller                     | hierarchy |
@@ -184,18 +184,14 @@ class PathManager:
         return self.path_manager.get(path)
     
     def update_path_manager(self, new_abspath):
-        """
-        该函数用于将导入的模块所在的绝对路径加入到路径管理器中.
-        """
+        """ 该函数用于将导入的模块所在的绝对路径加入到路径管理器中. """
         new_relpath = calculate_relative_path(self.launch_path, new_abspath)
         # relpath -> relative path
         self.path_manager[new_abspath] = new_relpath
 
 
 def calculate_relative_path(a, b):
-    """
-    已知两个绝对路径 a 和 b, 求 b 相对于 a 的相对路径.
-    """
+    """ 已知两个绝对路径 a 和 b, 求 b 相对于 a 的相对路径. """
     a, b = a.split('/'), b.split('/')
     
     intersection = -1
@@ -324,8 +320,8 @@ class AstAnalyser:
     def mask_text(self, text: str):
         """ 对文本中出现的汉字等双字节字符进行 "掩码" 处理.
         一个有意思的现象, 当 text 中含有汉字等非西文字符时, 会导致 ast 在解析时
-         的 col_offset 计算错误. 该问题在 https://bugs.python.org/issue21295 中
-         有所体现, 并得到了一些解释:
+        的 col_offset 计算错误. 该问题在 https://bugs.python.org/issue21295 中
+        有所体现, 并得到了一些解释:
             "...col_offset 是生成节点的第一个 token 的 utf-8 字节偏移."
         由于一个汉字占用两个字节, 就产生了 col_offset 偏移的问题.
         为了解决它, 我们需要将汉字转换成一个伪造的字母表示. 例如:
@@ -345,8 +341,8 @@ class AstAnalyser:
         try:
             root = ast_parse(fake_text)
         except SyntaxError:
-            """
-            这种情况发生的原因是, text 是不符合语法的代码. 例如:
+            """ 这种情况发生的原因是, text 是不符合语法的代码.
+            例如:
                 lk.logt(A, B
             B 的右侧没有括号封尾, 因此不符合语法规范. 将导致此项报错.
             遇到这种错误, 会返回 None. 请调用方继续处理 (认定为不可解析).
@@ -354,7 +350,7 @@ class AstAnalyser:
             return None
         
         # 利用 ast 对源码行 (fake_text) 中的各元素 "断句", ast 返回断句处的下标,
-        #  我们根据这些下标来分割 fake_text, 从而获取到各个独立的 "元素".
+        # 我们根据这些下标来分割 fake_text, 从而获取到各个独立的 "元素".
         # 根据测试, 第三层的遍历才是我们想要的正文.
         for a in iter_child_nodes(root):  # 1st layer: _ast.Expr (pass break)
             for b in iter_child_nodes(a):  # 2nd layer: _ast.Call (pass break)
@@ -413,15 +409,14 @@ class AstAnalyser:
 
 class LKLogger:
     """
-    注:
-        本类中的所有 h = 'self' 均指向 log 函数的直接调用者 (direct caller).
+    注: 本类中的所有 h = 'self' 均指向 log 函数的直接调用者 (direct caller).
         'parent' 均指向直接调用者的调用者, 'grand_parent' 是直接调用者的调用者的
-         调用者, 以此类推.
+        调用者, 以此类推.
         例如, 外部函数 fx() 调用了 LKLogger.log(), 则 fx() 是直接调用者.
         例如, 外部函数 fx() 调用了 LKLogger.over(), 而 over() 调用了
-         LKLogger.log(), 则 over() 是直接调用者, fx() 是间接调用者. (因此你可以
-         看到, 为了让指向到达外部函数 fx(), over() 函数中特意将层级参数上调为
-         'parent' 了)
+        LKLogger.log(), 则 over() 是直接调用者, fx() 是间接调用者. (因此你可以
+        看到, 为了让指向到达外部函数 fx(), over() 函数中特意将层级参数上调为
+        'parent' 了)
     """
     terminal = None  # None means terminal to console.
     
@@ -461,8 +456,7 @@ class LKLogger:
         self.recorder.record(self.path_manager.launch_path)
     
     def set_log_style(self, style: dict):
-        """
-        overwrite self.log_style configurations.
+        """ Overwrite self.log_style configurations.
         
         support properties:
             `show_func`: bool. default True.
@@ -488,20 +482,15 @@ class LKLogger:
     
     def _base_print(self, *msg, **kwargs):
         """
-        kwargs:
-            cnt
-            tag
+        kwargs.keys: cnt, tag
         """
         if self.log_enable is False:
             return
         
-        msg = ';\t'.join(msg)
+        msg = ';\t'.join(map(str, msg))
         
-        # | if self.terminal != 'console':
-        # | if isinstance(self.terminal, (staticmethod, classmethod)):
         if self.terminal is not None:
             return self.terminal(msg)
-        # else the terminal is console
         
         # personal settings (aka log style)
         cnt = kwargs.get('cnt', '')
@@ -528,15 +517,15 @@ class LKLogger:
         print(prefix, end='')
         """
         注意这里我们把 print 的 end 参数设为空字符串, 而由 prtend 控制是否在末尾
-         加空格. 这样做的目的是, 在 MsgRecorder 记录的时候, 也能记录到这个
-         prtend; 否则就会在 dump 的时候发现对齐不是很美观 (特别是涉及到 log*x,
-         logd, logt, logdt 的对齐的时候).
+        加空格. 这样做的目的是, 在 MsgRecorder 记录的时候, 也能记录到这个
+        prtend; 否则就会在 dump 的时候发现对齐不是很美观 (特别是涉及到 log*x,
+        logd, logt, logdt 的对齐的时候).
         """
         
         # strip r-char
         # msg = (i.replace('\r', '') for i in msg)  # 去除 \\r 的影响
         # 注: 去除 \\r 的方法暂时不用. 因为它会引起 self.recorder.record() 记录
-        #  失败的问题. 原因尚在调查中.
+        #   失败的问题. 原因尚在调查中.
         
         # print it
         print(msg)
@@ -546,15 +535,12 @@ class LKLogger:
     # --------------------------------------------------------------------------
     
     def log(self, *data, h='self'):
-        """
-        log (normal style).
-        """
-        _ = self._organize_messages(data, 'log', h)
+        """ log (normal style). """
+        self._organize_messages(data, 'log', h)
         return self._base_print(*data)
     
     def loga(self, *data, h='self'):
-        """
-        log in advanced mode.
+        """ log in advanced mode.
         
         NOTICE: 目前 (2019年7月7日) 无法解决以下问题:
             def foo(*x):
@@ -567,22 +553,19 @@ class LKLogger:
         return self._base_print(*msg)
     
     def logd(self, *data, style='-', length=64, h='self'):
-        """
-        log with divide line.
-        """
+        """ log with divide line. """
         msg = self._organize_messages(data, 'logd', h)
         return self._base_print(*msg, divide_line=style * length)
     
     def logt(self, tag, *data, h='self'):
-        """
-        log with tag.
+        """ log with tag.
 
         ARGS:
             tag: support list:
                 D, I, W, E, C
                 DEBUG, INFO, WARNING, ERROR, CRITICAL
                 D0323, I3234, W3145, E2345, C3634, ... (use live templates to
-                generate a timestamp (like `MMSS`) as a uniq code stamp.)
+                    generate a timestamp (like `MMSS`) as a uniq code stamp.)
             data
             h
         """
@@ -590,8 +573,7 @@ class LKLogger:
         return self._base_print(*msg, tag=tag)
     
     def logp(self, *data, h='self'):
-        """
-        log with pretty format.
+        """ log with pretty format.
         IN: [A, B, C]
         OT: [
                 A,
@@ -621,33 +603,25 @@ class LKLogger:
     # --------------------------------------------------------------------------
     
     def logx(self, *data, interval=1, h='self'):
-        """
-        log with index counter.
-        """
+        """ log with index counter. """
         if cnt := self._count_up(interval):
             self._organize_messages(data, 'logx', h)
             return self._base_print(*data, cnt=cnt)
     
     def logax(self, *data, interval=1, h='self'):
-        """
-        log in advanced with index counter.
-        """
+        """ log in advanced with index counter. """
         if cnt := self._count_up(interval):
             msg = self._organize_messages(data, 'logax', h)
             return self._base_print(*msg, cnt=cnt)
     
     def logdx(self, *data, style='-', length=64, interval=1, h='self'):
-        """
-        log with divide line and index counter.
-        """
+        """ log with divide line and index counter. """
         if cnt := self._count_up(interval):
             msg = self._organize_messages(data, 'logdx', h)
             return self._base_print(*msg, divide_line=style * length, cnt=cnt)
     
     def logtx(self, tag, *data, interval=1, h='self'):
-        """
-        log with tag and index counter.
-        """
+        """ log with tag and index counter. """
         if cnt := self._count_up(interval):
             msg = self._organize_messages(data, 'logtx', h)
             return self._base_print(*msg, tag=tag, cnt=cnt)
@@ -655,9 +629,7 @@ class LKLogger:
     # --------------------------------------------------------------------------
     
     def logdt(self, tag, *data, style='-', length=64, h='self'):
-        """
-        log with divide line and index counter.
-        """
+        """ log with divide line and index counter. """
         msg = self._organize_messages(data, 'logdt', h)
         return self._base_print(*msg, tag=tag, divide_line=style * length)
     
@@ -677,8 +649,8 @@ class LKLogger:
         self._denominator = self._measure_obj_length(x)
         return x
     
-    def init_count(self):
-        self.count(0)
+    def init_count(self, x=0):
+        return self.count(x)
     
     def enum(self, x, offset=0):
         self.counter = 0
@@ -795,7 +767,7 @@ class LKLogger:
     def _organize_messages(self, data: Tuple, caller: str,
                            h: Union[str, int]) -> tuple:
         """ Analyse source code line, extract and orgnize params' names and
-         values.
+        values.
         
         E.g.
             source code line: `lk.loga(name, gender, height, 'Engineer')`
@@ -843,7 +815,7 @@ class LKLogger:
                 if caller in ('logt', 'logtx', 'logdt', 'logdtx'):
                     # 这些 caller 的原型都是 logt.
                     # logt 有个特殊的地方在于, 它的第一个参数是 tag. 我们必须把
-                    #  它剔除.
+                    #   它剔除.
                     var_names.pop(0)
             
             node = self.code_tracker.setdefault(filepath, {})
@@ -852,8 +824,7 @@ class LKLogger:
             return var_names
     
     def _extract_vars_from_srcln(self, raw_line):
-        """
-        extract variants' names from source code line.
+        """ Extract variants' names from source code line.
         
         IN: source_code_line (str): e.g. 'lk.loga(a, b, c, "hello")  # comment'
         OT: list|None. e.g. ['a', 'b', 'c', '']
@@ -866,8 +837,8 @@ class LKLogger:
                _get_var_names() **cannot handle it**. we have to return a bad
                result (-> None)
                 
-        NOTE: this method will slow down the process, please use it as less as
-         possible.
+        NOTE: This method will slow down the process, please use it as less as
+            possible.
         """
         if not raw_line.startswith('lk.log'):
             if x := re.compile(r'lk\.log.+').findall(raw_line):
